'use strict';

const HintController = $scope => {

    const init = () => {
        $scope.advance = false;

        $scope.availableDistance = [
            {name: 'Very Close', value: 5},
            {name: 'Close', value: 25},
            {name: 'Middle', value: 50},
            {name: 'Far', value: 120},
            {name: 'Very Far', value: 250}
        ];

        chrome.storage.local.get(['hint', 'isFrameset'], result => {
            $scope.$applyAsync(() => {
                let hint = {
                    text: '',
                    maxDistance: 25,
                    matchCase: false,
                    fullMatch: false,
                    useIdAttribute: false
                };
                if (result.hint) {
                    hint = result.hint;
                }
                $scope.hint = hint;
                $scope.isFrameset = result.isFrameset;
            });
        });
    };

    init();

    $scope.hintKeyUpHandler = event => {
        if (event.keyCode === 13 && $scope.hint) {
            event.preventDefault();
            $scope.sendHint();
        }
    };

    $scope.sendHint = function () {
        chrome.storage.local.set({hint: $scope.hint});
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.HINT_DIALOG_RESULT,
            hint: $scope.hint,
            isFrameset: $scope.isFrameset
        });
    };

    $scope.closeDialog = skip => {
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.HINT_DIALOG_RESULT,
            skip,
            isFrameset: $scope.isFrameset
        });
    };

    $scope.advanceOptions = enabled => {
        $scope.advance = enabled;
        if (enabled) {
            chrome.windows.getCurrent(window => {
                if (window.height < CONSTANTS.DIALOG_OPTIONS.HINT.ADVANCED_HEIGHT) {
                    chrome.windows.update(window.id, {height: CONSTANTS.DIALOG_OPTIONS.HINT.ADVANCED_HEIGHT});
                }
            });
        } else {
            chrome.windows.getCurrent(window => {
                chrome.windows.update(window.id, {height: CONSTANTS.DIALOG_OPTIONS.HINT.HEIGHT});
            });
        }
    };
};

angular.module('gutenberg', []).controller('HintController', HintController);

HintController.$inject = ['$scope'];
