const handleMessage = (() => {
    const errorMessageBlock = document.getElementById('errorMessage');
    const hintText = document.getElementById('hintText');
    const advancedAhref = document.getElementById('advanced');
    const advancedAhrefJQ = $('#advanced');
    const advancedUpSpan = document.getElementById('advancedUp');
    const advancedDownSpan = document.getElementById('advancedDown');
    const advancedContainer = document.getElementById('advancedContainer');
    const maxDistance = $('#maxDistance');
    const matchCaseChkBox = document.getElementById('matchCase');
    const fullMatchChkBox = document.getElementById('fullMatch');
    const useIdAttributeChkBox = document.getElementById('useIdAttribute');
    const generateBtn = document.getElementById('generateBtn');
    const skipBtn = document.getElementById('skipBtn');
    const cancelBtn = document.getElementById('cancelBtn');
    let hint = {};
    let advanced = false;
    let isFrameset = false;
    let isDisabledHint = false;
    let isDisabledGenerating = false;

    const hintTextKeyUpHandler = event => {
        generateBtn.disabled = isDisabledHint ? isDisabledGenerating : !hintText.value;

        if (event.keyCode === 13 && hintText.value) {
            event.preventDefault();
            generateBtn.click();
        }
    };

    const fieldInputHandler = () => {
        generateBtn.disabled = isDisabledHint ? isDisabledGenerating : !hintText.value;
    };

    const openAdvanceOptionsHandler = () => {
        if (isDisabledHint) {
            return;
        }

        advanced = !advanced;
        updateAdvancedOptions();
        updateHeight();
    };

    const generateClickHandler = () => {
        sendHint();
    };

    const skipClickHandler = () => {
        closeDialog(true);
    };

    const cancelClickHandler = () => {
        closeDialog(false);
    };

    const sendHint = () => {
        hint = {
            text: hintText.value,
            maxDistance: Number.parseInt(maxDistance.val(), 10),
            matchCase: matchCaseChkBox.checked,
            usePatterns: false,
            fullMatch: fullMatchChkBox.checked,
            useIdAttribute: useIdAttributeChkBox.checked
        };
        chrome.storage.local.set({hint});
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.HINT_DIALOG_RESULT,
            hint,
            isFrameset,
            isDisabledHint
        });
    };

    const closeDialog = skip => {
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.HINT_DIALOG_RESULT,
            skip,
            isFrameset,
            isDisabledHint
        });
    };

    const updateAdvancedOptions = () => {
        advancedUpSpan.style.display = advanced ? '' : 'none';
        advancedDownSpan.style.display = !advanced ? '' : 'none';
        advancedContainer.style.display = advanced ? '' : 'none';
    };

    const updateHeight = () => {
        const overflowBefore = document.body.style.overflow;
        document.body.style.overflow = 'hidden';
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.SYNC_DIALOG_SIZE,
            dialogType: CONSTANTS.DIALOG_FRAME_OPTIONS.HINT.TYPE,
            dialogWidth: document.body.scrollWidth,
            dialogHeight: document.body.scrollHeight
        });
        document.body.style.overflow = overflowBefore;
    };

    return event => {
        if (!event.data.sender || event.data.sender !== CONSTANTS.MESSAGE_SENDERS.S7_OPEN_HINT_WINDOW) {
            return;
        }
        hint = event.data.hint ?
            event.data.hint :
            {
                text: '',
                maxDistance: 25,
                matchCase: false,
                usePatterns: false,
                fullMatch: false,
                useIdAttribute: false
            };
        isFrameset = event.data.isFrameset;
        advanced = false;
        isDisabledHint = event.data.isDisabledHint;
        isDisabledGenerating = event.data.isDisabledGenerating;

        errorMessageBlock.style.display = isDisabledHint ? '' : 'none';
        errorMessageBlock.innerText = isDisabledHint ?
            isDisabledGenerating ?
                CONSTANTS.ERROR_MESSAGES.XPATH_AND_CSS_CAN_NOT_BE_GENERATED :
                CONSTANTS.ERROR_MESSAGES.CUSTOM_CSS_SELECTOR_WILL_BE_GENERATED
            :
            '';
        hintText.disabled = isDisabledHint;
        hintText.value = hint.text;
        updateAdvancedOptions();
        maxDistance.val(hint.maxDistance).change();
        matchCaseChkBox.checked = hint.matchCase;
        fullMatchChkBox.checked = hint.fullMatch;
        useIdAttributeChkBox.checked = hint.useIdAttribute;
        generateBtn.disabled = isDisabledHint ? isDisabledGenerating : !hintText.value;

        if (isDisabledHint) {
            advancedAhrefJQ.addClass('disabled-href');
        } else {
            advancedAhrefJQ.removeClass('disabled-href');
        }

        if (isDisabledHint) {
            if (isDisabledGenerating) {
                cancelBtn.focus();
            } else {
                generateBtn.focus();
            }
        } else {
            hintText.focus();
        }

        hintText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_UP, hintTextKeyUpHandler);
        hintText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
        hintText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);
        advancedAhref.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, openAdvanceOptionsHandler);
        generateBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, generateClickHandler);
        skipBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, skipClickHandler);
        cancelBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, cancelClickHandler);

        updateHeight();
    }
})();
window.addEventListener(CONSTANTS.EVENTS.WINDOW.MESSAGE, handleMessage);
