const handleMessage = (() => {
    const locatorForm = document.getElementById('locatorForm');
    const errorMessageBlock = document.getElementById('errorMessage');
    const errorMessageBlockJQ = $('#errorMessage');
    const keyText = document.getElementById('key');
    const pageNameText = document.getElementById('pageName');
    const typeText = document.getElementById('type');
    const aliasText = document.getElementById('alias');
    const xpathText = document.getElementById('xpath');
    const cssText = document.getElementById('css');
    const frameText = document.getElementById('frame');
    const popupText = document.getElementById('popup');

    const copyXPathBtn = document.getElementById('copyXPath');
    const copyCssBtn = document.getElementById('copyCss');

    const pushToServerBtn = document.getElementById('pushToServerBtn');
    const testBtn = document.getElementById('testBtn');
    const continueBtn = document.getElementById('continueBtn');
    const backBtn = document.getElementById('backBtn');
    const cancelBtn = document.getElementById('cancelBtn');
    let locator = {};
    let hint;
    let saveFeatureEnabled = false;
    let validator;
    let isFrameset = false;
    let isDisabledHint = false;

    const formSubmitHandler = event => {
        event.preventDefault();
    };

    const isSpecialKey = keyCode => {
        switch (keyCode) {
            case 9:
            case 13:
            case 16:
            case 17:
            case 18:
            case 19:
            case 20:
            case 27:
            case 33:
            case 34:
            case 35:
            case 36:
            case 37:
            case 38:
            case 39:
            case 40:
            case 112:
            case 113:
            case 114:
            case 115:
            case 116:
            case 117:
            case 118:
            case 119:
            case 120:
            case 121:
            case 122:
            case 123:
            case 144:
            case 145:
                return true;
                break;
            default:
                return false;
                break;
        }
    };

    const fieldInputHandler = event => {
        if (isSpecialKey(event.which)) {
            return;
        }

        locator.pageName = pageNameText.value;
        locator.type = typeText.value;
        locator.alias = aliasText.value;
        locator.xpath = xpathText.value;
        locator.cssSelector = cssText.value;
        locator.frame = frameText.value;
        locator.popup = popupText.value;
        setKeyTextValue();
        pushToServerBtn.disabled = isLocatorFieldsNotFilled() || isCssAndXpathNotFilled();
        testBtn.disabled = isCssAndXpathNotFilled();
    };

    const pageNameKeyDownHandler = event => {
        if (isSpecialKey(event.which)) {
            return;
        }

        if (event.keyCode === 13) {
            event.preventDefault();
            typeText.focus();
        }
    };

    const typeKeyDownHandler = event => {
        if (isSpecialKey(event.which)) {
            return;
        }

        locator.type = typeText.value;
        setKeyTextValue();
        pushToServerBtn.disabled = isLocatorFieldsNotFilled() || isCssAndXpathNotFilled();

        if (event.keyCode === 13) {
            event.preventDefault();
            aliasText.focus();
        }
    };

    const aliasKeyDownHandler = event => {
        if (isSpecialKey(event.which)) {
            return;
        }

        locator.alias = aliasText.value;
        setKeyTextValue();
        pushToServerBtn.disabled = isLocatorFieldsNotFilled() || isCssAndXpathNotFilled();

        if (event.keyCode === 13) {
            event.preventDefault();
            xpathText.focus();
        }
    };

    const xpathKeyDownHandler = event => {
        if (isSpecialKey(event.which)) {
            return;
        }

        if (event.keyCode === 13) {
            event.preventDefault();
            cssText.focus();
        }
    };

    const xpathKeyUpHandler = event => {
        if (isSpecialKey(event.which)) {
            return;
        }

        locator.xpath = xpathText.value;
        pushToServerBtn.disabled = isLocatorFieldsNotFilled() || isCssAndXpathNotFilled();
        testBtn.disabled = isCssAndXpathNotFilled();
        copyXPathBtn.disabled = isEmptyField(xpathText.value);
    };

    const cssKeyDownHandler = event => {
        if (event.keyCode === 13) {
            event.preventDefault();
            frameText.focus();
        }
    };

    const cssKeyUpHandler = () => {
        if (isSpecialKey(event.which)) {
            return;
        }

        locator.cssSelector = cssText.value;
        pushToServerBtn.disabled = isLocatorFieldsNotFilled() || isCssAndXpathNotFilled();
        testBtn.disabled = isCssAndXpathNotFilled();
        copyCssBtn.disabled = isEmptyField(cssText.value);
    };

    const frameKeyDownHandler = event => {
        if (isSpecialKey(event.which)) {
            return;
        }

        locator.frame = frameText.value;

        if (event.keyCode === 13) {
            event.preventDefault();
            popupText.focus();
        }
    };

    const popupKeyDownHandler = event => {
        if (isSpecialKey(event.which)) {
            return;
        }

        locator.popup = popupText.value;

        if (event.keyCode === 13) {
            event.preventDefault();
            if (saveFeatureEnabled) {
                if (!(isLocatorFieldsNotFilled() || isCssAndXpathNotFilled())) {
                    pushToServerBtn.click();
                }
                pageNameText.focus();
            } else {
                if (!isCssAndXpathNotFilled()) {
                    testBtn.click();
                }
                xpathText.focus();
            }
        }
    };

    const copyXPathClickHandler = () => {
        copyToClipboard(xpathText);
    };

    const copyCssClickHandler = () => {
        copyToClipboard(cssText);
    };

    const copyToClipboard = element => {
        element.select();
        element.setSelectionRange(0, 99999);
        document.execCommand('copy');
        window.getSelection().removeAllRanges();
    };

    const pushToServerClickHandler = () => {
        if ($('#locatorForm').valid()) {
            chrome.storage.local.get('settings', result => {
                if (!result.settings) {
                    if (navigator.webdriver) {
                        showMessage(CONSTANTS.NOTIFICATION_MESSAGES.SETTINGS_NOT_FOUND);
                    } else {
                        alert(CONSTANTS.NOTIFICATION_MESSAGES.SETTINGS_NOT_FOUND);
                    }
                    return;
                }
                let savingLocator = Object.assign({}, locator);
                let selectors = getSelectorsToSend(result.settings.saveCssSelector);

                savingLocator.xpath = selectors.xpath;
                savingLocator.cssSelector = selectors.css;

                saveLocator(savingLocator, result.settings);
            });
        }
    };

    const testClickHandler = () => {
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.LOCATOR_DIALOG_RESULT,
            result: {
                type: CONSTANTS.ACTIONS.TEST,
                xpath: locator.xpath,
                cssSelector: locator.cssSelector,
                isFrameset
            }
        });
    };

    const continueClickHandler = () => {
        closeDialog(true);
    };

    const backClickHandler = () => {
        resetFormValidation();
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.LOCATOR_DIALOG_RESULT,
            result: {
                type: CONSTANTS.ACTIONS.BACK_TO_HINT,
                hint,
                isFrameset,
                isDisabledHint
            }
        });
    };

    const cancelClickHandler = () => {
        closeDialog(false);
    };

    const getSelectorsToSend = saveCssSelector => {
        let {xpath, cssSelector} = locator;

        if (((xpath !== null && xpath.length > 0) &&
                (cssSelector !== null && cssSelector.length > 0 && saveCssSelector)) ||
            ((xpath === null || xpath.length === 0) &&
                (cssSelector !== null && cssSelector.length > 0))) {
            return {
                xpath: null,
                css: cssSelector
            };
        }

        if (((xpath !== null && xpath.length > 0) &&
                (cssSelector !== null && cssSelector.length > 0 && !saveCssSelector)) ||
            ((cssSelector === null || cssSelector.length === 0) &&
                (xpath !== null && xpath.length > 0))) {
            return {
                xpath: xpath,
                css: null
            };
        }

        return {
            xpath: xpath,
            css: cssSelector
        };
    };

    const saveLocator = (savingLocator, settings) => {
        chrome.storage.local.set({locator: savingLocator});
        $("#overlay").fadeIn(300);

        let headers = settings.isBasicAuth ?
            {
                'Authorization': 'Basic ' + settings.auth
            } :
            {
                'X-API-KEY': settings.auth
            };

        $.ajax({
            url: settings.host + '/api/v1/locators',
            headers: headers,
            method: 'POST',
            contentType: "application/json; charset=utf-8",
            dataType: 'json',
            data: JSON.stringify(savingLocator),
            success: (data, textStatus, jqXHR) => {
                if (navigator.webdriver) {
                    showMessage(CONSTANTS.NOTIFICATION_MESSAGES.LOCATOR_SAVED);
                    closeDialog(true);
                } else {
                    setTimeout(() => {
                        $("#overlay").fadeOut(300);
                        alert(CONSTANTS.NOTIFICATION_MESSAGES.LOCATOR_SAVED);
                        closeDialog(true);
                    }, 500);
                }
            },
            error: (jqXHR, textStatus, errorThrown) => {
                if (navigator.webdriver) {
                    switch (jqXHR.status) {
                        case 400:
                            showMessage(jqXHR.responseJSON.message);
                            break;
                        case 401:
                            showMessage(CONSTANTS.NOTIFICATION_MESSAGES.WRONG_CREDENTIALS);
                            break;
                        case -1:
                            showMessage(CONSTANTS.NOTIFICATION_MESSAGES.WRONG_HOST);
                            break;
                        default:
                            showMessage(CONSTANTS.NOTIFICATION_MESSAGES.INTERNAL_SERVER_ERROR);
                            break;
                    }
                    pageNameText.focus();
                } else {
                    setTimeout(() => {
                        $("#overlay").fadeOut(300);
                        switch (jqXHR.status) {
                            case 400:
                                alert(jqXHR.responseJSON.message);
                                break;
                            case 401:
                                alert(CONSTANTS.NOTIFICATION_MESSAGES.WRONG_CREDENTIALS);
                                break;
                            case -1:
                                alert(CONSTANTS.NOTIFICATION_MESSAGES.WRONG_HOST);
                                break;
                            default:
                                alert(CONSTANTS.NOTIFICATION_MESSAGES.INTERNAL_SERVER_ERROR);
                                break;
                        }
                        pageNameText.focus();
                    }, 500);
                }
            },
            complete: () => {
                if (navigator.webdriver) {
                    setTimeout(() => {
                        $("#overlay").fadeOut(300);
                    }, 500);
                }
            }
        });
    };

    const closeDialog = skip => {
        resetFormValidation();
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.LOCATOR_DIALOG_RESULT,
            result: {
                type: CONSTANTS.ACTIONS.SKIP,
                skip,
                isFrameset
            }
        });
    };

    const resetFormValidation = () => {
        validator.errorList = [];
        validator.defaultShowErrors([]);
        validator.resetForm();
        validator.reset();
    };

    const updateHeight = () => {
        const overflowBefore = document.body.style.overflow;
        document.body.style.overflow = 'hidden';
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.SYNC_DIALOG_SIZE,
            dialogType: CONSTANTS.DIALOG_FRAME_OPTIONS.LOCATOR.TYPE,
            dialogWidth: document.body.scrollWidth,
            dialogHeight: document.body.scrollHeight
        });
        document.body.style.overflow = overflowBefore;
    };

    const setKeyTextValue = () => {
        keyText.value = (pageNameText.value ? pageNameText.value + '.' : '.') +
            (typeText.value ? typeText.value + '.' : '.') +
            (aliasText.value ? aliasText.value : '');
    };

    const setTextValue = (textElement, value) => {
        textElement.value = value ? value : '';
    };

    const isCssAndXpathNotFilled = () => {
        return isDisabledHint ?
            isEmptyField(cssText.value) :
            isEmptyField(xpathText.value) && isEmptyField(cssText.value);
    };

    const isLocatorFieldsNotFilled = () => {
        return isEmptyField(pageNameText.value) || isEmptyField(typeText.value) || isEmptyField(aliasText.value);
    };

    const isEmptyField = value => {
        return value === null || typeof value === 'undefined' || value.length === 0;
    };

    const showMessage = string => {
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.SEND_NOTIFICATION,
            id: CONSTANTS.NOTIFICATION_IDS.LOCATOR,
            message: string
        });
    };

    const setValidationOptions = () => {
        $.validator.addMethod('pattern', function (value, element, params) {
            const expression = new RegExp(params);
            return this.optional(element) || expression.test(value);
        });

        const options = {
            rules: {
                pageName: getRulesForKey(),
                type: getRulesForKey(),
                alias: getRulesForKey(),
            },
            messages: {
                pageName: {
                    required: 'Web Page is required',
                    pattern: 'Web Page may not contain any white spaces, dashes, dots etc. Use \'_\' as separator instead'
                },
                type: {
                    required: 'Type is required',
                    pattern: 'Type may not contain any white spaces, dashes, dots etc. Use \'_\' as separator instead'
                },
                alias: {
                    required: 'Alias is required',
                    pattern: 'Alias may not contain any white spaces, dashes, dots etc. Use \'_\' as separator instead'
                },
            },
            tooltip_options: {
                pageName: getDefaultTooltipOptions(),
                type: getDefaultTooltipOptions(),
                alias: getDefaultTooltipOptions()
            },
            errorClass: 'has-error',
            highlight: (element, errorClass) => {
                $(element.parentElement).addClass(errorClass);
            },
            unhighlight: (element, errorClass) => {
                $(element.parentElement).removeClass(errorClass);
            }
        };

        validator = $('#locatorForm').validate(options);
    };

    const getRulesForKey = () => {
        return {
            required: {
                depends: () => {
                    return saveFeatureEnabled;
                }
            },
            pattern: {
                depends: () => {
                    return saveFeatureEnabled;
                },
                param: /^[_a-zA-Z0-9]+$/,
            }
        }
    };

    const getDefaultTooltipOptions = () => {
        return {
            trigger: 'focus',
            placement: 'bottom',
            delay: {
                'hide': 300
            }
        };
    };

    return event => {
        if (!event.data.sender || event.data.sender !== CONSTANTS.MESSAGE_SENDERS.S7_OPEN_LOCATOR_WINDOW) {
            return;
        }

        if (event.data.locator) {
            locator = event.data.locator;
        }
        isDisabledHint = event.data.isDisabledHint;
        try {
            if (!isDisabledHint) {
                locator.cssSelector = xPathToCss(locator.xpath);
            }
            copyCssBtn.disabled = false;
        } catch (error) {
            console.debug(`${new Date(Date.now()).toISOString()} -> ${error.message}`);
            locator.cssSelector = null;
            copyCssBtn.disabled = true;
        }
        hint = event.data.hint;
        locator.alias = hint.text
            .replace(/[\. ]/, '_')
            .replace(/[^\w]/gi, '')
            .toLowerCase();
        saveFeatureEnabled = event.data.saveFeatureEnabled;
        isFrameset = event.data.isFrameset;

        errorMessageBlock.style.display = isDisabledHint ? '' : 'none';
        errorMessageBlock.innerText = isDisabledHint ? CONSTANTS.ERROR_MESSAGES.CUSTOM_CSS_SELECTOR_WILL_BE_GENERATED : '';
        if (!saveFeatureEnabled) {
            errorMessageBlockJQ.addClass('alert-locator');
        } else {
            errorMessageBlockJQ.removeClass('alert-locator');
        }
        xpathText.disabled = isDisabledHint;


        setTextValue(pageNameText, locator.pageName);
        setTextValue(typeText, locator.type);
        setTextValue(aliasText, locator.alias);
        setTextValue(xpathText, locator.xpath);
        setTextValue(cssText, locator.cssSelector);
        setTextValue(frameText, locator.frame);
        setTextValue(popupText, locator.popup);
        setKeyTextValue();

        keyText.style.display = saveFeatureEnabled ? '' : 'none';
        pageNameText.style.display = saveFeatureEnabled ? '' : 'none';
        typeText.style.display = saveFeatureEnabled ? '' : 'none';
        aliasText.style.display = saveFeatureEnabled ? '' : 'none';
        pushToServerBtn.style.display = saveFeatureEnabled ? '' : 'none';

        keyText.disabled = true;
        pushToServerBtn.disabled = isLocatorFieldsNotFilled() || isCssAndXpathNotFilled();
        testBtn.disabled = isCssAndXpathNotFilled();

        copyXPathBtn.disabled = isDisabledHint;

        setValidationOptions();

        if (saveFeatureEnabled) {
            pageNameText.focus();
        } else {
            xpathText.focus();
        }

        locatorForm.addEventListener(CONSTANTS.EVENTS.DOM.SUBMIT, formSubmitHandler);

        pageNameText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, pageNameKeyDownHandler);
        pageNameText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
        pageNameText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);

        typeText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, typeKeyDownHandler);
        typeText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
        typeText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);

        aliasText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, aliasKeyDownHandler);
        aliasText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
        aliasText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);

        xpathText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, xpathKeyDownHandler);
        xpathText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_UP, xpathKeyUpHandler);
        xpathText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
        xpathText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);

        cssText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, cssKeyDownHandler);
        cssText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_UP, cssKeyUpHandler);
        cssText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
        cssText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);

        frameText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, frameKeyDownHandler);
        frameText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
        frameText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);

        popupText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, popupKeyDownHandler);
        popupText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
        popupText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);

        copyXPathBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, copyXPathClickHandler);
        copyCssBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, copyCssClickHandler);

        pushToServerBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, pushToServerClickHandler);
        testBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, testClickHandler);
        continueBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, continueClickHandler);
        backBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, backClickHandler);
        cancelBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, cancelClickHandler);

        updateHeight();
    }
})();
window.addEventListener(CONSTANTS.EVENTS.WINDOW.MESSAGE, handleMessage);
