document.addEventListener(CONSTANTS.EVENTS.WINDOW.DOM_CONTENT_LOADED, () => {
    const settingsForm = document.getElementById('settingsForm');
    const usernameContainer = document.getElementById('usernameContainer');
    const passwordContainer = document.getElementById('passwordContainer');
    const apikeyContainer = document.getElementById('apikeyContainer');
    const hostText = document.getElementById('host');
    const basicAuthRadio = document.getElementById('basicAuthRadio');
    const apiKeyRadio = document.getElementById('apiKeyRadio');
    const pasteApiKeyBtn = document.getElementById('pasteApiKey');
    const usernameText = document.getElementById('username');
    const passwordText = document.getElementById('password');
    const apikeyText = document.getElementById('apikey');
    const cssSelectorChkBox = document.getElementById('cssSelector');

    const saveBtn = document.getElementById('saveBtn');
    const testConnectionBtn = document.getElementById('testConnectionBtn');
    const resetBtn = document.getElementById('resetBtn');
    let isBasicAuth = true;
    let apiKey = null;

    const initForm = () => {
        chrome.storage.local.get('settings', (result) => {
            if (result.settings) {
                isBasicAuth = result.settings.isBasicAuth;
                apiKey = result.settings.apikey;
                resetBtn.style.display = '';

                let credentials = isBasicAuth ? atob(result.settings.auth) : apiKey;

                setTextValue(hostText, result.settings.host);
                if (isBasicAuth) {
                    setTextValue(usernameText, result.settings.username);
                    setTextValue(passwordText, credentials.substr(credentials.indexOf(':') + 1));
                    apikeyText.value = null;
                } else {
                    usernameText.value = null;
                    passwordText.value = null;
                    apikeyText.value = processApiKeyView(credentials);
                }

                cssSelectorChkBox.checked = result.settings.saveCssSelector;
            } else {
                resetBtn.style.display = 'none';
            }

            basicAuthRadio.checked = isBasicAuth;
            apiKeyRadio.checked = !isBasicAuth;
            setFieldsVisibility();
            setButtonsState();

            setValidationOptions();

            hostText.focus();
        });
    };

    const formSubmitHandler = event => {
        event.preventDefault();
    };

    const fieldInputHandler = event => {
        saveBtn.disabled = isRequiredFieldsNotFilled();
        testConnectionBtn.disabled = isRequiredFieldsNotFilled();
    };

    const hostKeyDownHandler = event => {
        saveBtn.disabled = isRequiredFieldsNotFilled();
        testConnectionBtn.disabled = isRequiredFieldsNotFilled();

        if (event.keyCode === 13) {
            event.preventDefault();
            if (isBasicAuth) {
                usernameText.focus();
            } else {
                apikeyText.focus();
            }
        }
    };

    const basicAuthRadioClickHandler = event => {
        isBasicAuth = true;
        apiKeyRadio.checked = false;
        setFieldsVisibility();
        setButtonsState();
    };

    const apiKeyRadioClickHandler = event => {
        isBasicAuth = false;
        basicAuthRadio.checked = false;
        setFieldsVisibility();
        setButtonsState();
    };

    const setFieldsVisibility = () => {
        usernameContainer.style.display = isBasicAuth ? '' : 'none';
        passwordContainer.style.display = isBasicAuth ? '' : 'none';
        apikeyContainer.style.display = isBasicAuth ? 'none' : '';
    };

    const setButtonsState = () => {
        saveBtn.disabled = isRequiredFieldsNotFilled();
        testConnectionBtn.disabled = isRequiredFieldsNotFilled();
    };

    const usernameKeyDownHandler = event => {
        setButtonsState();

        if (event.keyCode === 13) {
            event.preventDefault();
            passwordText.focus();
        }
    };

    const passwordKeyDownHandler = event => {
        if (event.keyCode === 13) {
            event.preventDefault();
            cssSelectorChkBox.focus();
        }
    };

    const apikeyKeyDownHandler = event => {
        setButtonsState();
    };

    const processApiKeyView = value => {
        if (value.length === 0 || value.length === 1 || value.length === 2) {
            return value;
        }
        if (value.length > 2) {
            return value[0] + '*'.repeat(40) + value[value.length - 1];
        }
    };

    const cssSelectorKeyDownHandler = () => {
        if (event.keyCode === 13) {
            event.preventDefault();
            if (!isRequiredFieldsNotFilled()) {
                saveBtn.focus();
            } else {
                hostText.focus();
            }
        }
    };

    const pasteApiKeyClickHandler = () => {
        apikeyText.removeAttribute(CONSTANTS.ATTRIBUTES.READONLY);
        apikeyText.focus();
        apikeyText.select();
        document.execCommand('paste');
        apiKey = apikeyText.value;
        apikeyText.value = processApiKeyView(apikeyText.value);
        fieldInputHandler();
        apikeyText.setAttribute(CONSTANTS.ATTRIBUTES.READONLY, 'readonly');
        apikeyText.blur();
    };

    const saveClickHandler = () => {
        if ($('#settingsForm').valid()) {
            let savingSettings = {
                host: normalizeURL(hostText.value),
                isBasicAuth: isBasicAuth,
                username: isBasicAuth ? usernameText.value : '',
                apikey: !isBasicAuth ? apiKey : '',
                auth: getAuth(),
                saveCssSelector: cssSelectorChkBox.checked
            };

            chrome.storage.local.set(
                {
                    settings: savingSettings,
                    saveFeatureEnabled: true
                },
                () => {
                    if (navigator.webdriver) {
                        showMessage(CONSTANTS.NOTIFICATION_MESSAGES.CONFIGURATION_SAVED);
                    } else {
                        alert(CONSTANTS.NOTIFICATION_MESSAGES.CONFIGURATION_SAVED);
                    }
                    initForm();
                });
        }
    };

    const testConnectionClickHandler = () => {
        $("#overlay").fadeIn(300);

        let headers = isBasicAuth ?
            {
                'Authorization': 'Basic ' + getAuth(),
                'X-Requested-By': 'XMLHttpRequest',
            } :
            {
                'X-API-KEY': getAuth(),
                'X-Requested-By': 'XMLHttpRequest',
            };

        $.ajax({
            url: normalizeURL(hostText.value) + '/api/v2/health-check',
            headers: headers,
            method: 'GET',
            contentType: "application/json; charset=utf-8",
            success: (data, textStatus, jqXHR) => {
                if (navigator.webdriver) {
                    if (jqXHR.status === 204) {
                        showMessage(CONSTANTS.NOTIFICATION_MESSAGES.CONNECTION_TEST_SUCCESSFUL);
                    } else {
                        showMessage(CONSTANTS.NOTIFICATION_MESSAGES.UNEXPECTED_RESPONSE
                            .replace('$STATUS', jqXHR.status).replace('$DATA', data)
                        );
                    }
                } else {
                    setTimeout(() => {
                        $("#overlay").fadeOut(300);
                        if (jqXHR.status === 204) {
                            alert(CONSTANTS.NOTIFICATION_MESSAGES.CONNECTION_TEST_SUCCESSFUL);
                        } else {
                            alert(CONSTANTS.NOTIFICATION_MESSAGES.UNEXPECTED_RESPONSE
                                .replace('$STATUS', jqXHR.status).replace('$DATA', data)
                            );
                        }
                    }, 500);
                }
            },
            error: (jqXHR, textStatus, errorThrown) => {
                if (navigator.webdriver) {
                    switch (jqXHR.status) {
                        case -1:
                            showMessage(CONSTANTS.NOTIFICATION_MESSAGES.WRONG_HOST_CONFIGURATION);
                            break;
                        case 401:
                            showMessage(
                                isBasicAuth ?
                                    CONSTANTS.NOTIFICATION_MESSAGES.WRONG_CREDENTIALS :
                                    CONSTANTS.NOTIFICATION_MESSAGES.WRONG_CREDENTIALS_API_KEY
                            );
                            break;
                        default:
                            showMessage(CONSTANTS.NOTIFICATION_MESSAGES.UNEXPECTED_RESPONSE
                                .replace('$STATUS', jqXHR.status).replace('$DATA', jqXHR.data)
                            );
                            break;
                    }
                } else {
                    setTimeout(() => {
                        $("#overlay").fadeOut(300);
                        switch (jqXHR.status) {
                            case -1:
                                alert(CONSTANTS.NOTIFICATION_MESSAGES.WRONG_HOST_CONFIGURATION);
                                break;
                            case 401:
                                alert(CONSTANTS.NOTIFICATION_MESSAGES.WRONG_CREDENTIALS);
                                break;
                            default:
                                alert(CONSTANTS.NOTIFICATION_MESSAGES.UNEXPECTED_RESPONSE
                                    .replace('$STATUS', jqXHR.status).replace('$DATA', jqXHR.data)
                                );
                                break;
                        }
                    }, 500);
                }
            },
            complete: () => {
                if (navigator.webdriver) {
                    setTimeout(() => {
                        $("#overlay").fadeOut(300);
                    }, 500);
                }
            }
        });
    };

    const resetClickHandler = () => {
        resetBtn.style.display = 'none';
        hostText.value = null;
        usernameText.value = null;
        passwordText.value = null;
        apikeyText.value = null;
        apiKey = null;
        basicAuthRadio.checked = true;
        basicAuthRadioClickHandler();
        cssSelectorChkBox.checked = false;
        chrome.storage.local.remove(['settings', 'saveFeatureEnabled']);
    };

    const getAuth = () => {
        return isBasicAuth ?
            btoa(usernameText.value + ':' + passwordText.value) :
            apiKey;
    };

    const normalizeURL = url => {
        if (!url.startsWith('https://') && !url.startsWith('http://')) {
            url = "https://" + url;
        }
        try {
            return new URL(url).origin;
        } catch (error) {
            console.debug(`${new Date(Date.now()).toISOString()} -> Invalid url provided: ${error}`);
            return url;
        }
    };

    const showMessage = string => {
        chrome.runtime.sendMessage({
            action: CONSTANTS.ACTIONS.SEND_NOTIFICATION,
            id: CONSTANTS.NOTIFICATION_IDS.OPTIONS,
            message: string
        });
    };

    const setValidationOptions = () => {
        const options = {
            rules: {
                host: {
                    required: true,
                    url2: true
                },
                username: {
                    required: {
                        depends: () => {
                            return isBasicAuth;
                        }
                    }
                },
                apikey: {
                    required: {
                        depends: () => {
                            return !isBasicAuth;
                        }
                    }
                }
            },
            messages: {
                host: {
                    required: 'Host is required',
                    url2: 'Please enter a valid URL'
                },
                username: {
                    required: 'Username is required'
                },
                apikey: {
                    required: 'API Key is required'
                }
            },
            tooltip_options: {
                host: getDefaultTooltipOptions(),
                username: getDefaultTooltipOptions(),
                apikey: getDefaultTooltipOptions(),
            },
            errorClass: 'has-error',
            highlight: (element, errorClass) => {
                $(element.parentElement).addClass(errorClass);
            },
            unhighlight: (element, errorClass) => {
                $(element.parentElement).removeClass(errorClass);
            }
        };

        // Same as url, but TLD is optional
        $.validator.addMethod(
            "url2",
            function (value, element) {
                return this.optional(element) || /^(?:(?:(?:https?|ftp):)?\/\/)(?:(?:[^\]\[?\/<~#`!@$^&*()+=}|:";',>{ ]|%[0-9A-Fa-f]{2})+(?::(?:[^\]\[?\/<~#`!@$^&*()+=}|:";',>{ ]|%[0-9A-Fa-f]{2})*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z0-9\u00a1-\uffff][a-z0-9\u00a1-\uffff_-]{0,62})?[a-z0-9\u00a1-\uffff]\.)+(?:[a-z\u00a1-\uffff]{2,}\.?)|(?:(?:[a-z0-9\u00a1-\uffff][a-z0-9\u00a1-\uffff_-]{0,62})?[a-z0-9\u00a1-\uffff])|(?:(?:[a-z0-9\u00a1-\uffff][a-z0-9\u00a1-\uffff_-]{0,62}\.)))(?::\d{2,5})?(?:[/?#]\S*)?$/i.test(value);
            },
            $.validator.messages.url
        );

        $('#settingsForm').validate(options);
    };

    const getDefaultTooltipOptions = () => {
        return {
            trigger: 'focus',
            placement: 'bottom',
            delay: {
                'hide': 300
            }
        };
    };

    const setTextValue = (textElement, value) => {
        if (value) {
            textElement.value = value;
        }
    };

    const isRequiredFieldsNotFilled = () => {
        return isBasicAuth ?
            isEmptyField(hostText.value) || isEmptyField(usernameText.value) :
            isEmptyField(hostText.value) || isEmptyField(apikeyText.value);
    };

    const isEmptyField = value => {
        return value === null || typeof value === 'undefined' || value.length === 0;
    };

    initForm();

    settingsForm.addEventListener(CONSTANTS.EVENTS.DOM.SUBMIT, formSubmitHandler);

    hostText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, hostKeyDownHandler);
    hostText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
    hostText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);
    basicAuthRadio.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, basicAuthRadioClickHandler);
    apiKeyRadio.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, apiKeyRadioClickHandler);
    usernameText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, usernameKeyDownHandler);
    usernameText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
    usernameText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);
    passwordText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, passwordKeyDownHandler);
    apikeyText.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, apikeyKeyDownHandler);
    apikeyText.addEventListener(CONSTANTS.EVENTS.DOM.INPUT, fieldInputHandler);
    apikeyText.addEventListener(CONSTANTS.EVENTS.DOM.BLUR, fieldInputHandler);
    cssSelectorChkBox.addEventListener(CONSTANTS.EVENTS.DOM.KEY_DOWN, cssSelectorKeyDownHandler);

    pasteApiKeyBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, pasteApiKeyClickHandler);
    saveBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, saveClickHandler);
    testConnectionBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, testConnectionClickHandler);
    resetBtn.addEventListener(CONSTANTS.EVENTS.DOM.CLICK, resetClickHandler);
});
