let targetTabId = null;
let welcomeDialogTabId = null;
let responseReceived = false;
let isNavigatorWebdriver = false;
let dialogTabId = null;

importScripts(
    'constants.js',
);

const keepServiceRunning = () => {
    setTimeout(keepServiceRunning, CONSTANTS.KEEP_ALIVE_CHECK_TIMEOUT);
};

keepServiceRunning();

chrome.storage.local.set({
    locator: {},
    hint: {
        text: '',
        maxDistance: 25,
        matchCase: false,
        usePatterns: false,
        fullMatch: false,
        useIdAttribute: false
    }
});

const deleteTimer = port => {
    if (port._timer) {
        clearTimeout(port._timer);
        delete port._timer;
    }
};

chrome.runtime.onConnect.addListener(portObject => {
    if (portObject.name !== CONSTANTS.MESSAGING_PORT) {
        return;
    }

    portObject.onMessage.addListener((msg, port) => {
        console.debug(`${new Date(Date.now()).toISOString()} -> [KEEP_ALIVE] Port '${port.name}' is open and connected`);
        console.debug(`${new Date(Date.now()).toISOString()} -> [MESSAGE_EXCHANGE] Received ${JSON.stringify(msg)} from ${JSON.stringify(port.sender)}`);

        isNavigatorWebdriver = msg.isAutomationBrowser;

        if (!isNavigatorWebdriver) {
            chrome.storage.local.get('locatorExplorerInitFlag', (storage) => {
                if (!storage.locatorExplorerInitFlag) {
                    openWelcomeDialog();
                    chrome.storage.local.set({
                        locatorExplorerInitFlag: true,
                    });
                }
            });
        }
    });
    portObject.onDisconnect.addListener(deleteTimer);
    portObject._timer = setTimeout(
        port => {
            deleteTimer(port);
            console.debug(`${new Date(Date.now()).toISOString()} -> [KEEP_ALIVE] Port '${port.name}' is closed and disconnected by Service Worker`);
            port.disconnect();
        },
        CONSTANTS.KEEP_ALIVE_CHECK_TIMEOUT,
        portObject
    );
});

chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    let action = request.action;
    switch (action) {
        case CONSTANTS.ACTIONS.ELEMENT_CAPTURED:
            let {isFrameset, isDisabledHint, isDisabledGenerating} = request;
            chrome.storage.local.get('hint', storage => {
                if (storage.hint) {
                    storage.hint.text = '';
                    chrome.storage.local.set({hint: storage.hint}, () => {
                        if (isFrameset) {
                            chrome.storage.local.set(
                                {
                                    isFrameset
                                },
                                () => {
                                    openHintDialog();
                                }
                            );
                        } else {
                            openHintFrameDialog({
                                hint: storage.hint,
                                isFrameset,
                                isDisabledHint,
                                isDisabledGenerating
                            });
                        }
                    });
                } else {
                    if (isFrameset) {
                        chrome.storage.local.set(
                            {
                                hint: null,
                                isFrameset
                            },
                            () => {
                                openHintDialog();
                            }
                        );
                    } else {
                        openHintFrameDialog({
                            hint: null,
                            isFrameset,
                            isDisabledHint,
                            isDisabledGenerating
                        });
                    }
                }
            });
            break;
        case CONSTANTS.ACTIONS.HINT_DIALOG_RESULT:
            processHintDialogResult(request);
            break;
        case CONSTANTS.ACTIONS.LOCATOR_DIALOG_RESULT:
            processLocatorDialogResult(request.result);
            break;
        case CONSTANTS.ACTIONS.TEST_XPATH_FINISHED:
            chrome.tabs.sendMessage(targetTabId, {
                action: CONSTANTS.ACTIONS.TEST_CSS,
                css: request.css,
                isFrameset: request.isFrameset
            });
            break;
        case CONSTANTS.ACTIONS.TEST_CSS_FINISHED:
            if (request.isFrameset) {
                focusTab(dialogTabId);
            } else {
                chrome.tabs.sendMessage(targetTabId, {
                    action: CONSTANTS.ACTIONS.SHOW_DIALOG,
                    dialogType: CONSTANTS.DIALOG_FRAME_OPTIONS.LOCATOR.TYPE
                });
            }
            break;
        case CONSTANTS.ACTIONS.GENERATION_FINISHED:
            if (request.result.hasError) {
                if (request.isFrameset) {
                    chrome.storage.local.set(
                        {
                            hint: request.hint,
                            isFrameset: request.isFrameset
                        },
                        () => {
                            openHintDialog();
                        }
                    );
                } else {
                    openHintFrameDialog({
                        hint: request.hint,
                        isFrameset: request.isFrameset,
                        isDisabledHint: false
                    });
                }
            } else {
                chrome.storage.local.get('locator', (storage) => {
                    let locator = {};
                    if (storage.locator) {
                        locator = storage.locator;
                    }
                    locator.xpath = request.result.xpath;
                    locator.cssSelector = request.result.css;
                    locator.frame = request.result.frame;
                    locator.popup = '';
                    chrome.storage.local.set({locator: locator}, () => {
                        if (request.isFrameset) {
                            chrome.storage.local.set(
                                {
                                    locator,
                                    hint: request.hint,
                                    isFrameset: request.isFrameset
                                },
                                () => {
                                    openLocatorDialog();
                                }
                            );
                        } else {
                            openLocatorFrameDialog({
                                locator,
                                hint: request.hint,
                                isFrameset: request.isFrameset,
                                isDisabledHint: request.isDisabledHint
                            });
                        }
                    });
                });
            }
            break;
        case CONSTANTS.ACTIONS.CLOSE_WELCOME_DIALOG:
            closeWelcomeDialog();
            break;
        case CONSTANTS.ACTIONS.FOCUS_DIALOG:
            focusTab(targetTabId);
            break;
        case CONSTANTS.ACTIONS.SYNC_DIALOG_SIZE: {
            chrome.tabs.sendMessage(targetTabId, request);
            break;
        }
        case CONSTANTS.ACTIONS.SEND_NOTIFICATION:
            const {id, message} = request;
            showNotification(id, message);
            break;
        default:
            break;
    }
});

chrome.runtime.onMessageExternal.addListener((request, sender, sendResponse) => {
    if (sender.id === CONSTANTS.EXTERNAL_EXTENSIONS.RECORDER.ID || sender.id === CONSTANTS.EXTERNAL_EXTENSIONS.RECORDER.ID_FROM_STORE) {
        if (request.reason === CONSTANTS.EXTERNAL_EXTENSIONS.RECORDER.REASON) {
            if (targetTabId) {
                stopCapturing(true);
            }
            sendResponse();
            return true;
        }
    }
    sendResponse();
    return true;
});

chrome.tabs.onActivated.addListener((event) => {
    if (event.tabId !== dialogTabId) {
        setTimeout(() => {
            changeIcon(targetTabId && (event.tabId === targetTabId));
        }, 500);
    }
});

chrome.tabs.onUpdated.addListener((tabId, changeInfo) => {
    if (changeInfo.status === 'complete' && targetTabId && tabId === targetTabId) {
        chrome.tabs.sendMessage(tabId, {
            action: CONSTANTS.ACTIONS.START_CAPTURING
        });
    }
});

chrome.tabs.onRemoved.addListener((tabId) => {
    if (targetTabId && tabId === targetTabId) {
        targetTabId = null;
        changeIcon(false);
    }
});

chrome.action.onClicked.addListener((tab) => {
    responseReceived = false;

    closeWelcomeDialog();

    if (targetTabId && tab.id !== targetTabId) {
        chrome.tabs.sendMessage(targetTabId, {action: CONSTANTS.ACTIONS.STOP_CAPTURING}, () => {
            changeIcon(false);
            startCapturing(tab.id);
        });
    } else {
        if (targetTabId) {
            stopCapturing(false);
        } else {
            chrome.management.get(
                CONSTANTS.EXTERNAL_EXTENSIONS.RECORDER.ID,
                answer => {
                    chrome.runtime.sendMessage(
                        !!answer ?
                            CONSTANTS.EXTERNAL_EXTENSIONS.RECORDER.ID :
                            CONSTANTS.EXTERNAL_EXTENSIONS.RECORDER.ID_FROM_STORE,
                        {
                            reason: CONSTANTS.EXTERNAL_EXTENSIONS.RECORDER.REASON
                        },
                        null,
                        (response) => {
                            if (response) {
                                if (response.isEnabled) {
                                    responseReceived = true;
                                    showMessage(CONSTANTS.NOTIFICATION_IDS.BACKGROUND, CONSTANTS.NOTIFICATION_MESSAGES.CANNOT_ENABLE_GUTENBERG);
                                } else {
                                    startCapturing(tab.id);
                                }
                            } else {
                                startCapturing(tab.id);
                            }
                        });
                }
            )
        }
    }
    setTimeout(() => {
        if (!responseReceived) {
            showMessage(CONSTANTS.NOTIFICATION_IDS.BACKGROUND, CONSTANTS.NOTIFICATION_MESSAGES.REFRESH_PAGE);
        }
    }, 1500)
});

const startCapturing = tabId => {
    chrome.tabs.sendMessage(tabId, {action: CONSTANTS.ACTIONS.START_CAPTURING}, (response) => {
        responseReceived = true;
        if (!response || !response.started) {
            showMessage(CONSTANTS.NOTIFICATION_IDS.BACKGROUND, CONSTANTS.NOTIFICATION_MESSAGES.REFRESH_PAGE);
            return;
        }
        targetTabId = tabId;
        changeIcon(true);
    });
};

const stopCapturing = showNotification => {
    chrome.tabs.sendMessage(targetTabId, {action: CONSTANTS.ACTIONS.STOP_CAPTURING}, (response) => {
        responseReceived = true;
        if (!response || response.started) {
            showMessage(CONSTANTS.NOTIFICATION_IDS.BACKGROUND, CONSTANTS.NOTIFICATION_MESSAGES.REFRESH_PAGE);
            return;
        }
        targetTabId = null;
        changeIcon(false);
        if (showNotification) {
            showMessage(CONSTANTS.NOTIFICATION_IDS.BACKGROUND, CONSTANTS.NOTIFICATION_MESSAGES.GUTENBERG_IS_INACTIVE);
        }
    });
};

const changeIcon = enabled => {
    chrome.action.setIcon({
        path: enabled ?
            '/img/app-icon128x128.png' :
            '/img/app-disabled.png'
    });
    chrome.action.setTitle({
        title: enabled ?
            CONSTANTS.EXTENSION_NAME.concat(' Enabled') :
            'Click to enable '.concat(CONSTANTS.EXTENSION_NAME)
    });
};

const processHintDialogResult = request => {
    if (request.isFrameset) {
        closeDialog();
    } else {
        closeFrameDialog(CONSTANTS.DIALOG_FRAME_OPTIONS.HINT.TYPE);
    }
    if (request.hint) {
        chrome.storage.local.get('patterns', (storage) => {
            request.hint.patterns = storage.patterns;
            chrome.tabs.sendMessage(targetTabId, {
                action: CONSTANTS.ACTIONS.GENERATE_SMART_XPATH,
                hint: request.hint,
                isDisabledHint: request.isDisabledHint
            });
        })
    } else {
        chrome.tabs.sendMessage(targetTabId, {
            action: CONSTANTS.ACTIONS.RESET_CAPTURING,
            fireEvent: request.skip
        });
    }
};

const processLocatorDialogResult = result => {
    switch (result.type) {
        case CONSTANTS.ACTIONS.BACK_TO_HINT:
            if (result.isFrameset) {
                closeDialog();
                chrome.storage.local.set(
                    {
                        hint: result.hint,
                        isFrameset: result.isFrameset
                    },
                    () => {
                        openHintDialog();
                    }
                );
            } else {
                closeFrameDialog(CONSTANTS.DIALOG_FRAME_OPTIONS.LOCATOR.TYPE);
                openHintFrameDialog({
                    hint: result.hint,
                    isFrameset: result.isFrameset,
                    isDisabledHint: result.isDisabledHint
                });
            }
            break;
        case CONSTANTS.ACTIONS.TEST:
            if (result.isFrameset) {
                focusTab(targetTabId);
            } else {
                chrome.tabs.sendMessage(targetTabId, {
                    action: CONSTANTS.ACTIONS.HIDE_DIALOG,
                    dialogType: CONSTANTS.DIALOG_FRAME_OPTIONS.LOCATOR.TYPE
                });
            }
            chrome.tabs.sendMessage(targetTabId, {
                action: CONSTANTS.ACTIONS.TEST_XPATH,
                xpath: result.xpath,
                css: result.cssSelector,
                isFrameset: result.isFrameset
            });
            break;
        case CONSTANTS.ACTIONS.SKIP:
            if (result.isFrameset) {
                closeDialog();
            } else {
                closeFrameDialog(CONSTANTS.DIALOG_FRAME_OPTIONS.LOCATOR.TYPE);
            }
            chrome.tabs.sendMessage(targetTabId, {
                action: CONSTANTS.ACTIONS.RESET_CAPTURING,
                fireEvent: result.skip
            });
            break;
        default:
            break;
    }
};

const openWelcomeDialog = () => {
    welcomeDialogTabId = null;

    chrome.windows.getCurrent(window => {
        let screenDimensions =
            {
                width: window.width,
                height: window.height
            };
        chrome.windows.create({
                url: CONSTANTS.DIALOG_OPTIONS.WELCOME.URL,
                type: 'popup',
                width: CONSTANTS.DIALOG_OPTIONS.WELCOME.WIDTH,
                height: CONSTANTS.DIALOG_OPTIONS.WELCOME.HEIGHT,
                left: Math.round((screenDimensions.width / 2) - (CONSTANTS.DIALOG_OPTIONS.WELCOME.WIDTH / 2)),
                top: Math.round((screenDimensions.height / 2) - (CONSTANTS.DIALOG_OPTIONS.WELCOME.HEIGHT / 2)),
                focused: false
            },
            window => {
                welcomeDialogTabId = window.tabs[0].id;
                focusTab(welcomeDialogTabId);
            });
    });
};

const closeWelcomeDialog = () => {
    if (welcomeDialogTabId !== null) {
        chrome.tabs.get(welcomeDialogTabId, (tab) => {
            welcomeDialogTabId = null;
            chrome.windows.remove(tab.windowId);
        });
    }
};

const openHintFrameDialog = options => {
    openFrameDialog(CONSTANTS.DIALOG_FRAME_OPTIONS.HINT.TYPE, options);
};

const openLocatorFrameDialog = options => {
    chrome.storage.local.get(['saveFeatureEnabled'], storage => {
        options.saveFeatureEnabled = storage.saveFeatureEnabled;
        openFrameDialog(CONSTANTS.DIALOG_FRAME_OPTIONS.LOCATOR.TYPE, options);
    });
};

const openFrameDialog = (dialogType, options) => {
    chrome.tabs.sendMessage(targetTabId, {
        action: CONSTANTS.ACTIONS.OPEN_DIALOG,
        dialogType,
        options
    });
};

const closeFrameDialog = dialogType => {
    chrome.tabs.sendMessage(targetTabId, {
        action: CONSTANTS.ACTIONS.CLOSE_DIALOG,
        dialogType
    });
};

const openHintDialog = () => {
    openDialog(
        CONSTANTS.DIALOG_OPTIONS.HINT.URL,
        CONSTANTS.DIALOG_OPTIONS.HINT.WIDTH,
        CONSTANTS.DIALOG_OPTIONS.HINT.HEIGHT
    );
};

const openLocatorDialog = () => {
    chrome.storage.local.get(['saveFeatureEnabled'], storage => {
        let height = storage.saveFeatureEnabled ?
            CONSTANTS.DIALOG_OPTIONS.LOCATOR.ADVANCED_HEIGHT :
            CONSTANTS.DIALOG_OPTIONS.LOCATOR.HEIGHT;

        openDialog(
            CONSTANTS.DIALOG_OPTIONS.LOCATOR.URL,
            CONSTANTS.DIALOG_OPTIONS.LOCATOR.WIDTH,
            height
        );
    });
};

const openDialog = (url, width, height) => {
    dialogTabId = null;

    chrome.windows.getCurrent(window => {
        let screenDimensions =
            {
                width: window.width,
                height: window.height
            };
        chrome.windows.create(
            {
                url: url,
                type: 'popup',
                width: width,
                height: height,
                left: Math.round((screenDimensions.width / 2) - (width / 2)),
                top: Math.round((screenDimensions.height / 2) - (height / 2)),
                focused: false
            },
            window => {
                dialogTabId = window.tabs[0].id;
                focusTab(dialogTabId);
            }
        );
    });
};

const closeDialog = () => {
    if (dialogTabId !== null) {
        chrome.tabs.get(dialogTabId, tab => {
            dialogTabId = null;
            chrome.windows.remove(tab.windowId);
        });
    }
};

const focusTab = tabId => {
    chrome.tabs.get(tabId, (tab) => {
        chrome.windows.update(tab.windowId, {'focused': true});
        chrome.tabs.update(tab.id, {'active': true});
        changeIcon(true);
    });
};

const showNotification = (id, message) => {
    chrome.notifications.clear(CONSTANTS.NOTIFICATION_IDS.BACKGROUND, () => {
        chrome.notifications.clear(CONSTANTS.NOTIFICATION_IDS.CONTENT, () => {
            chrome.notifications.clear(CONSTANTS.NOTIFICATION_IDS.LOCATOR, () => {
                chrome.notifications.clear(CONSTANTS.NOTIFICATION_IDS.OPTIONS, () => {
                    chrome.notifications.create(id,
                        {
                            type: 'basic',
                            iconUrl: CONSTANTS.NOTIFICATION_ICON_URL,
                            title: CONSTANTS.EXTENSION_NAME,
                            message
                        });
                });
            });
        });
    });
};

const showMessage = (id, message) => {
    if (isNavigatorWebdriver) {
        showNotification(id, message);
    } else {
        chrome.tabs.query({active: true}, tabs => {
            chrome.tabs.sendMessage(tabs[0].id, {
                action: CONSTANTS.ACTIONS.SHOW_ALERT,
                message
            });
        });
    }
};
